package sensor

import (
	"encoding/json"
	"git.loafle.net/overflow/commons_go/model/timestamp"
	"git.loafle.net/overflow/overflow_service/proxy/meta"
	"git.loafle.net/overflow/overflow_service/proxy/target"
	"git.loafle.net/overflow/overflow_service/proxy/utils"
)

type Sensor struct {
	Id                json.Number            `json:"id,Number,omitempty"`
	CreateDate        timestamp.Timestamp    `json:"createDate,omitempty"`
	Description       string                 `json:"description,omitempty"`
	MetaSensorStatus  *meta.MetaSensorStatus `json:"status,omitempty"`
	Target            *target.Target         `json:"target,omitempty"`
	Crawler           *meta.MetaCrawler      `json:"crawler,omitempty"`
	CrawlerInputItems string                 `json:"crawlerInputItems,omitempty"`
}

type SensorService struct {
}

func NewSensorService() *SensorService {
	return &SensorService{}
}

func (ss *SensorService) Regist(s *Sensor) (string, error) {
	out, err := utils.InvokeDBByModel("sensor", "save", s, utils.MODEL_SENSOR)

	if err != nil {
		return "", err
	}

	return out, nil
}

func (ss *SensorService) ReadAllByTarget(t *target.Target) (string, error) {
	out, err := utils.InvokeDBByModel("sensor", "findAllByTarget", t, utils.MODEL_TARGET)

	if err != nil {
		return "", err
	}

	return out, err
}

func (ss *SensorService) Read(id string) (string, error) {
	out, err := utils.InvokeDBByModel("sensor", "findOne", id, utils.MODEL_LONG)

	if err != nil {
		return "", err
	}

	return out, nil
}
